defmodule LinkShortenerWeb.Api.V1.AccountsController do
  use LinkShortenerWeb, :controller

  alias LinkShortener.Accounts
  alias LinkShortener.Accounts.User
  alias LinkShortenerWeb.Auth.Guardian

  action_fallback LinkShortenerWeb.FallbackController

  def sign_up(conn, %{"user" => user_params}) do
    with {:ok, %User{} = user} <- Accounts.register_user(user_params),
	 {:ok, token, _claims} <- Guardian.encode_and_sign(user) do
      conn
      |> put_status(:created)
      |> render(:user, %{user: user, token: token})
    end
  end

  def sign_in(conn, %{"email" => email, "password" => password}) do
    with {:ok, user, token} <- Guardian.authenticate(email, password) do
      conn
      |> put_status(:created)
      |> render(:user, %{user: user, token: token})
    end
  end

  def sign_out(conn, %{}) do
    token = Guardian.Plug.current_token(conn)
    Guardian.revoke(token)
    conn
    |> put_status(:ok)
    |> render(:sign_out, %{token: token})
  end
end
