defmodule LinkShortenerWeb.LinkLive.Index do
  use LinkShortenerWeb, :live_view

  alias LinkShortener.Links
  alias LinkShortener.Links.Link

  @impl true
  def mount(_params, _session, socket) do
    current_user = socket.assigns.current_user
    is_random_shorten_by_default = true

    {:ok,
     socket
     |> stream(:links, Links.get_all_by_user(current_user))
     |> assign(:is_random_shorten_by_default, is_random_shorten_by_default)}
  end

  @impl true
  def handle_params(params, _url, socket) do
    {:noreply, apply_action(socket, socket.assigns.live_action, params)}
  end

  defp apply_action(socket, :edit, %{"id" => id}) do
    socket
    |> assign(:page_title, "Edit Link")
    |> assign(:link, Links.get_one!(id))
  end

  defp apply_action(socket, :new, _params) do
    socket
    |> assign(:page_title, "New Link")
    |> assign(:link, %Link{})
  end

  defp apply_action(socket, :index, _params) do
    socket
    |> assign(:page_title, "Listing Links")
    |> assign(:link, nil)
  end

  @impl true
  def handle_info({LinkShortenerWeb.LinkLive.FormComponent, {:saved, link}}, socket) do
    {:noreply, stream_insert(socket, :links, link)}
  end

  @impl true
  def handle_event("delete", %{"id" => id}, socket) do
    link = Links.get_one!(id)
    {:ok, _} = Links.delete_one(link)

    {:noreply, stream_delete(socket, :links, link)}
  end
end
